/*
 * main.c Torn Desktop Demo (c) copyright Mike Wynn 2001 [03 Oct 2001]
 */

#include "gba.h"
#include "gba_io.h"
#include "gba_sprites.h"
#include "gba_font.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

extern signed short cosTable[360];
extern signed short sinTable[360];

static unsigned char * digits = "0123456789ABCDEF";

static signed short getSin( int angle )
{
	while( angle < 0 ) angle+=360;
	return sinTable[angle%360];
}

static signed short getCos( int angle )
{
	while( angle < 0 ) angle+=360;
	return cosTable[angle%360];
}

static void
printNumber( int num, int x, int y, int palette )
{
	unsigned char tmp[6];
	tmp[0] = digits[(num>>12)&0xF];
	tmp[1] = digits[(num>>8)&0xF];
	tmp[2] = '.';
	tmp[3] = digits[(num>>4)&0xF];
	tmp[4] = digits[num&0xF];
	tmp[5] = 0;
	msgPrint( 0, x, y, palette, tmp ); 
}

static void
printInt( int num, int x, int y, int palette )
{
	unsigned char tmp[5];
	tmp[0] = digits[(num>>12)&0xF];
	tmp[1] = digits[(num>>8)&0xF];
	tmp[2] = digits[(num>>4)&0xF];
	tmp[3] = digits[num&0xF];
	tmp[4] = 0;
	msgPrint( 0, x, y, palette, tmp ); 
}

static void 
blockPrint( int x, int y, int w, int h, int val, int palette )
{
	int xx;
	w = x+w;
	h = y+h;
	xx = x;
	while ( y < h )
	{
		x = xx;
		while( x < w )
		{
			tilePut( 0, x, y, palette, val );
			x++;
		}
		y++;
	}
}

static void
createSolidTiles()
{
	HalfwordPtr tileBase;
	int val;

	tileBase = VRAM_BASE + 0x2000; // (this is a halfword offset so would be 0x4000 byte offset)
	int i, j;
	for( i = 15; i >= 0; i-- )
	{
		val = (i|i<<4|i<<8|i<<12);
		for( j = 15; j >= 0; j-- )
		{
			tileBase[j+i*16] = val;
		}
	}
}

void matrixMultiply( int * dest, int * one, int * two )
{
	int a;
	int b;
	int c;
	int d;

	a = ((one[0]*two[0])>>8)+((one[1]*two[2])>>8);
	b = ((one[0]*two[1])>>8)+((one[1]*two[3])>>8);
	c = ((one[2]*two[0])>>8)+((one[3]*two[2])>>8);
	d = ((one[2]*two[1])>>8)+((one[3]*two[3])>>8);

	dest[0] = a;
	dest[1] = b;
	dest[2] = c;
	dest[3] = d;
}

void createShearXMatrix( int * mat, int xshear )
{
	mat[0] = 256;
	mat[1] = xshear;
	mat[2] = 0;
	mat[3] = 256;
}

void createShearYMatrix( int * mat, int yshear )
{
	mat[0] = 256;
	mat[1] = 0;
	mat[2] = yshear;
	mat[3] = 256;
}

void createScaleMatrix( int * mat, int xscale, int yscale )
{
	mat[0] = xscale;
	mat[1] = 0;
	mat[2] = 0;
	mat[3] = yscale;
}

void createRotateMatrix( int * mat, int angle )
{
	mat[0] = getCos( angle );
	mat[1] = getSin( angle );
	mat[2] = -getSin( angle );
	mat[3] = getCos( angle );
}

static int matrixes[4] = { 0, 1, 2, 3 };
static int mat0[4];
static int mat1[4];
static int mat2[4];
static int mat3[4];
static int result[4];
int main()
{
	int angle  = 0;
	int scalex = 256;
	int scaley = 256;
	int shearx = 0;
	int sheary = 0;
	int repeatdelay =  0;
	int repeatwait  = 10;
	int tmp;


	wait4Vsync(); // wait until its safe to begin writing to vRam
	initialiseBG( 0, 0x0000, 0x4000, BG_TEXT_32x32, BG_COLOUR_16 );

	zeroOam();

	clearMap( 0x0000, 1024*16, 0 ); /* clear all vram */
	/* font data only starts at tile 32 */
	loadTilesFromRom( 0x4000+(32*32), l8fontData, 32*16*32 );
	loadSpriteDataFromRom( l8spriteData );
	loadBGPalette( l8fontPalette, 0, 256 );
	loadOBJPalette( l8palette, 0, 256 );
	createSolidTiles();

	/* enable BG0 and Objects (sprites) */
	initialiseMode( 0, 1, 0, 0, 0, 0, 0 );
//                          000000000011111111112222222222
//                          012345678901234567890123456789
	msgPrint( 0,  0,  4, 0, "Rotation Scale Shear test app" ); 
	msgPrint( 0,  0,  5, 0, "Rotate:" ); 
	msgPrint( 0,  0,  6, 0, "  top buttons" ); 
	msgPrint( 0,  0,  7, 0, "Scale: (direction pad + A)" ); 
	msgPrint( 0,  0,  8, 0, "Shear: (direction pad + B)" ); 
	msgPrint( 0,  0,  9, 0, "swap first matrixes (start)" ); 
	msgPrint( 0,  0, 10, 0, "rotate the order   (select)" ); 
	msgPrint( 0,  0, 11, 0, "A and B together to reset" ); 
	msgPrint( 0,  0, 12, 0, "without any buttons:" ); 
	msgPrint( 0,  0, 13, 0, "left  - zoom in" ); 
	msgPrint( 0,  0, 14, 0, "right - zoom out" ); 
	msgPrint( 0,  0, 15, 0, "up    - reset shear params" ); 
	msgPrint( 0,  0, 16, 0, "down  - reset zoom  params" ); 


	wait4Vsync(); // wait until its safe to begin writing to vRam

	while( !readKeys() );
//	while( readKeys() );

	/* enable BG0 and Objects (sprites) */
	initialiseMode( 0, 1, 0, 0, 0, 1, 0 );

	OBJ_DATA[0].y    = (16 | OBJ_ROTATE | OBJ_SDOUBLE);
	OBJ_DATA[0].x    = (56 | OBJ_SIZE_64); // rs offset of 0;
	OBJ_DATA[0].tile =  (6<<12)|272;

	OBJ_ROT_DATA[0].rotParamA = 256;
	OBJ_ROT_DATA[0].rotParamB = 0;
	OBJ_ROT_DATA[0].rotParamC = 0;
	OBJ_ROT_DATA[0].rotParamD = 256;

	blockPrint(  0,  0, 30, 20,  0, 0 );
	blockPrint(  7,  2, 16, 16,  8, 0 );
	blockPrint( 11,  6,  8,  8, 15, 0 );

	while(1)
	{
		int keys;
		wait4Vsync(); // wait until its safe to begin writing to vRam

		switch( matrixes[0] )
		{
		case 0:
			createRotateMatrix( mat0, angle );
			msgPrint( 0, 0, 19, 0, "Rotate" ); 
			break;
		case 1:
			createScaleMatrix(  mat0, scalex, scaley );
			msgPrint( 0, 0, 19, 0, "Scale " ); 
			break;
		case 2:
			createShearXMatrix(  mat0, shearx );
			msgPrint( 0, 0, 19, 0, "ShearX" ); 
			break;
		case 3:
			createShearYMatrix(  mat0, sheary );
			msgPrint( 0, 0, 19, 0, "ShearY" ); 
			break;
		}
		switch( matrixes[1] )
		{
		case 0:
			createRotateMatrix( mat1, angle );
			msgPrint( 0, 8, 19, 0, "Rotate" ); 
			break;
		case 1:
			createScaleMatrix(  mat1, scalex, scaley );
			msgPrint( 0, 8, 19, 0, "Scale " ); 
			break;
		case 2:
			createShearXMatrix(  mat1, shearx );
			msgPrint( 0, 8, 19, 0, "ShearX" ); 
			break;
		case 3:
			createShearYMatrix(  mat1, sheary );
			msgPrint( 0, 8, 19, 0, "ShearY" ); 
			break;
		}
		switch( matrixes[2] )
		{
		case 0:
			createRotateMatrix( mat2, angle );
			msgPrint( 0, 15, 19, 0, "Rotate" ); 
			break;
		case 1:
			createScaleMatrix(  mat2, scalex, scaley );
			msgPrint( 0, 15, 19, 0, "Scale " ); 
			break;
		case 2:
			createShearXMatrix( mat2, shearx );
			msgPrint( 0, 15, 19, 0, "ShearX" ); 
			break;
		case 3:
			createShearYMatrix( mat2, sheary );
			msgPrint( 0, 15, 19, 0, "ShearY" ); 
			break;
		}
		switch( matrixes[3] )
		{
		case 0:
			createRotateMatrix( mat3, angle );
			msgPrint( 0, 22, 19, 0, "Rotate" ); 
			break;
		case 1:
			createScaleMatrix(  mat3, scalex, scaley );
			msgPrint( 0, 22, 19, 0, "Scale " ); 
			break;
		case 2:
			createShearXMatrix( mat3, shearx );
			msgPrint( 0, 22, 19, 0, "ShearX" ); 
			break;
		case 3:
			createShearYMatrix( mat3, sheary );
			msgPrint( 0, 22, 19, 0, "ShearY" ); 
			break;
		}

		matrixMultiply( result, mat0, mat1 );
		matrixMultiply( result, result, mat2 );
		matrixMultiply( result, result, mat3 );

		msgPrint( 0, 0, 5, 0, "A" ); 
		msgPrint( 0, 0, 6, 0, "B" ); 
		msgPrint( 0, 0, 7, 0, "C" ); 
		msgPrint( 0, 0, 8, 0, "D" ); 

		printNumber( result[0], 2, 5, 0 );
		printNumber( result[1], 2, 6, 0 );
		printNumber( result[2], 2, 7, 0 );
		printNumber( result[3], 2, 8, 0 );

		msgPrint( 0, 0, 0, 0, "Scale X: 00.00 Scale Y: 00.00" );
		msgPrint( 0, 0, 1, 0, "Shear X: 00.00 Shear Y: 00.00" );

		printNumber( scalex,  9, 0, 0 );
		printNumber( scaley, 24, 0, 0 );
		printNumber( shearx,  9, 1, 0 );
		printNumber( sheary, 24, 1, 0 );

		msgPrint( 0, 0, 2, 0, "Rotate" );
		printInt( angle, 2, 3, 0 );

		OBJ_ROT_DATA[0].rotParamA = result[0];
		OBJ_ROT_DATA[0].rotParamB = result[1];
		OBJ_ROT_DATA[0].rotParamC = result[2];
		OBJ_ROT_DATA[0].rotParamD = result[3];
		
		if ( repeatdelay > 0 ) 
		{
			repeatdelay--;
			continue;
		}
		keys = readKeys();
		if ( keys )
		{
			repeatdelay = repeatwait;
			if ( repeatwait > 0 ) repeatwait--;
			if ( keys & PAD_BUTTONA )
			{
				if ( keys & PAD_BUTTONB )
				{
					angle  = 0;
					scalex = 256;
					scaley = 256;
					shearx = 0;
					sheary = 0;
					matrixes[0] = 0;
					matrixes[1] = 1;
					matrixes[2] = 2;
				}
				// scale;
				if ( keys & PAD_UP    ) scaley++;
				if ( keys & PAD_DOWN  ) scaley--;
				if ( keys & PAD_LEFT  ) scalex--;
				if ( keys & PAD_RIGHT ) scalex++;
			}
			if ( keys & PAD_BUTTONB )
			{
				// shear;
				if ( keys & PAD_UP    ) sheary++;
				if ( keys & PAD_DOWN  ) sheary--;
				if ( keys & PAD_LEFT  ) shearx--;
				if ( keys & PAD_RIGHT ) shearx++;
			}
			if ( !(keys & (PAD_BUTTONA | PAD_BUTTONB)) )
			{
				if ( keys & PAD_UP    ) { shearx = 0; sheary = 0; }
				if ( keys & PAD_DOWN  ) { scalex = 256; scaley = 256; }
				if ( keys & PAD_LEFT  ) { scalex--; scaley--; }
				if ( keys & PAD_RIGHT ) { scalex++; scaley++; }
			}
			if ( keys & PAD_BUTTONL ) angle--;
			if ( keys & PAD_BUTTONR ) angle++;
			if ( keys & PAD_START )
			{
				tmp         = matrixes[0];
				matrixes[0] = matrixes[1];
				matrixes[1] = tmp;
			}
			if ( keys & PAD_SELECT )
			{
				tmp         = matrixes[0];
				matrixes[0] = matrixes[1];
				matrixes[1] = matrixes[2];
				matrixes[2] = matrixes[3];
				matrixes[3] = tmp;
			}
		}
		else
		{
			repeatwait = 10;
		}
	}
}

signed short sinTable[360] = {
0,
4,
8,
13,
17,
22,
26,
31,
35,
40,
44,
48,
53,
57,
61,
66,
70,
74,
79,
83,
87,
91,
95,
100,
104,
108,
112,
116,
120,
124,
128,
131,
135,
139,
143,
146,
150,
154,
157,
161,
164,
167,
171,
174,
177,
181,
184,
187,
190,
193,
196,
198,
201,
204,
207,
209,
212,
214,
217,
219,
221,
223,
226,
228,
230,
232,
233,
235,
237,
238,
240,
242,
243,
244,
246,
247,
248,
249,
250,
251,
252,
252,
253,
254,
254,
255,
255,
255,
255,
255,
255,
255,
255,
255,
255,
255,
254,
254,
253,
252,
252,
251,
250,
249,
248,
247,
246,
244,
243,
242,
240,
238,
237,
235,
233,
232,
230,
228,
226,
223,
221,
219,
217,
214,
212,
209,
207,
204,
201,
198,
196,
193,
190,
187,
184,
181,
177,
174,
171,
167,
164,
161,
157,
154,
150,
146,
143,
139,
135,
131,
127,
124,
120,
116,
112,
108,
104,
100,
95,
91,
87,
83,
79,
74,
70,
66,
61,
57,
53,
48,
44,
40,
35,
31,
26,
22,
17,
13,
8,
4,
0,
65532,
65528,
65523,
65519,
65514,
65510,
65505,
65501,
65496,
65492,
65488,
65483,
65479,
65475,
65470,
65466,
65462,
65457,
65453,
65449,
65445,
65441,
65436,
65432,
65428,
65424,
65420,
65416,
65412,
65408,
65405,
65401,
65397,
65393,
65390,
65386,
65382,
65379,
65375,
65372,
65369,
65365,
65362,
65359,
65355,
65352,
65349,
65346,
65343,
65340,
65338,
65335,
65332,
65329,
65327,
65324,
65322,
65319,
65317,
65315,
65313,
65310,
65308,
65306,
65304,
65303,
65301,
65299,
65298,
65296,
65294,
65293,
65292,
65290,
65289,
65288,
65287,
65286,
65285,
65284,
65284,
65283,
65282,
65282,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65282,
65282,
65283,
65284,
65284,
65285,
65286,
65287,
65288,
65289,
65290,
65292,
65293,
65294,
65296,
65298,
65299,
65301,
65303,
65304,
65306,
65308,
65310,
65313,
65315,
65317,
65319,
65322,
65324,
65327,
65329,
65332,
65335,
65338,
65340,
65343,
65346,
65349,
65352,
65355,
65359,
65362,
65365,
65369,
65372,
65375,
65379,
65382,
65386,
65390,
65393,
65397,
65401,
65405,
65409,
65412,
65416,
65420,
65424,
65428,
65432,
65436,
65441,
65445,
65449,
65453,
65457,
65462,
65466,
65470,
65475,
65479,
65483,
65488,
65492,
65496,
65501,
65505,
65510,
65514,
65519,
65523,
65528,
65532
};

static signed short cosTable[360] = {
256,
255,
255,
255,
255,
255,
254,
254,
253,
252,
252,
251,
250,
249,
248,
247,
246,
244,
243,
242,
240,
238,
237,
235,
233,
232,
230,
228,
226,
223,
221,
219,
217,
214,
212,
209,
207,
204,
201,
198,
196,
193,
190,
187,
184,
181,
177,
174,
171,
167,
164,
161,
157,
154,
150,
146,
143,
139,
135,
131,
127,
124,
120,
116,
112,
108,
104,
100,
95,
91,
87,
83,
79,
74,
70,
66,
61,
57,
53,
48,
44,
40,
35,
31,
26,
22,
17,
13,
8,
4,
0,
65532,
65528,
65523,
65519,
65514,
65510,
65505,
65501,
65496,
65492,
65488,
65483,
65479,
65475,
65470,
65466,
65462,
65457,
65453,
65449,
65445,
65441,
65436,
65432,
65428,
65424,
65420,
65416,
65412,
65408,
65405,
65401,
65397,
65393,
65390,
65386,
65382,
65379,
65375,
65372,
65369,
65365,
65362,
65359,
65355,
65352,
65349,
65346,
65343,
65340,
65338,
65335,
65332,
65329,
65327,
65324,
65322,
65319,
65317,
65315,
65313,
65310,
65308,
65306,
65304,
65303,
65301,
65299,
65298,
65296,
65294,
65293,
65292,
65290,
65289,
65288,
65287,
65286,
65285,
65284,
65284,
65283,
65282,
65282,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65282,
65282,
65283,
65284,
65284,
65285,
65286,
65287,
65288,
65289,
65290,
65292,
65293,
65294,
65296,
65298,
65299,
65301,
65303,
65304,
65306,
65308,
65310,
65313,
65315,
65317,
65319,
65322,
65324,
65327,
65329,
65332,
65335,
65338,
65340,
65343,
65346,
65349,
65352,
65355,
65359,
65362,
65365,
65369,
65372,
65375,
65379,
65382,
65386,
65390,
65393,
65397,
65401,
65405,
65409,
65412,
65416,
65420,
65424,
65428,
65432,
65436,
65441,
65445,
65449,
65453,
65457,
65462,
65466,
65470,
65475,
65479,
65483,
65488,
65492,
65496,
65501,
65505,
65510,
65514,
65519,
65523,
65528,
65532,
0,
4,
8,
13,
17,
22,
26,
31,
35,
40,
44,
48,
53,
57,
61,
66,
70,
74,
79,
83,
87,
91,
95,
100,
104,
108,
112,
116,
120,
124,
128,
131,
135,
139,
143,
146,
150,
154,
157,
161,
164,
167,
171,
174,
177,
181,
184,
187,
190,
193,
196,
198,
201,
204,
207,
209,
212,
214,
217,
219,
221,
223,
226,
228,
230,
232,
233,
235,
237,
238,
240,
242,
243,
244,
246,
247,
248,
249,
250,
251,
252,
252,
253,
254,
254,
255,
255,
255,
255,
255
};


