/*
 * main.c Torn Desktop Demo (c) copyright Mike Wynn 2001 [03 Oct 2001]
 */

#include "gba.h"
#include "gba_io.h"
#include "gba_sprites.h"
#include "gba_font.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

static int param[4];
static unsigned char * digits = "0123456789ABCDEF";

static void
printNumber( int num, int x, int y, int palette )
{
	unsigned char tmp[6];
	tmp[0] = digits[(num>>12)&0xF];
	tmp[1] = digits[(num>>8)&0xF];
	tmp[2] = '.';
	tmp[3] = digits[(num>>4)&0xF];
	tmp[4] = digits[num&0xF];
	tmp[5] = 0;
	msgPrint( 0, x, y, palette, tmp ); 
}

int main()
{
	int activeParam = 0;
	int tmp = 0;

	wait4Vsync(); // wait until its safe to begin writing to vRam
	initialiseBG( 0, 0x0000, 0x4000, BG_TEXT_32x32, BG_COLOUR_16 );

	zeroOam();

	clearMap( 0x0000, 1024*16, 0 ); /* clear all vram */
	/* font data only starts at tile 32 */
	loadTilesFromRom( 0x4000+(32*32), l8fontData, 32*16*32 );
	loadSpriteDataFromRom( l8spriteData );
	loadBGPalette( l8fontPalette, 0, 256 );
	loadOBJPalette( l8palette, 0, 256 );

	/* enable BG0 and Objects (sprites) */
	initialiseMode( 0, 1, 0, 0, 0, 1, 0 );

	msgPrint( 0, 1, 16, 0, "hello world!" ); 

	msgPrint( 0, 1, 17, 0, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" ); 
	msgPrint( 0, 1, 18, 0, "abcdefghijklmnopqrstuvwxyz" ); 
	msgPrint( 0, 1, 19, 0, "!@#$%^&*_+|~-=\\`" );
	msgPrint( 0, 1, 20, 0, "(){}[]<>:;'\",./?" ); 

	OBJ_DATA[0].y    = (48 | OBJ_ROTATE | OBJ_SDOUBLE);
	OBJ_DATA[0].x    = (88 | OBJ_SIZE_64); // rs offset of 0;
	OBJ_DATA[0].tile =  (6<<12)|272;


	// initialise as no scale no rotate

	OBJ_ROT_DATA[0].rotParamA = param[0] = 256;
	OBJ_ROT_DATA[0].rotParamB = param[1] = 0;
	OBJ_ROT_DATA[0].rotParamC = param[2] = 0;
	OBJ_ROT_DATA[0].rotParamD = param[3] = 256;


	wait4Vsync(); // wait until its safe to begin writing to vRam

	while(1)
	{
		int keys;
		wait4Vsync(); // wait until its safe to begin writing to vRam

		msgPrint( 0, 2, 4, (activeParam == 0)?0:1, "rot A                   " ); 
		msgPrint( 0, 2, 5, (activeParam == 1)?0:1, "Rot B                   " ); 
		msgPrint( 0, 2, 6, (activeParam == 2)?0:1, "Rot C                   " ); 
		msgPrint( 0, 2, 7, (activeParam == 3)?0:1, "Rot D                   " ); 
		printNumber( param[0], 8, 4, (activeParam == 0)?0:1 );
		printNumber( param[1], 8, 5, (activeParam == 1)?0:1 );
		printNumber( param[2], 8, 6, (activeParam == 2)?0:1 );
		printNumber( param[3], 8, 7, (activeParam == 3)?0:1 );

		OBJ_ROT_DATA[0].rotParamA = param[0];
		OBJ_ROT_DATA[0].rotParamB = param[1];
		OBJ_ROT_DATA[0].rotParamC = param[2];
		OBJ_ROT_DATA[0].rotParamD = param[3];
		
		keys = readKeys();
		if ( keys )
		{
			if ( keys & PAD_UP )
			{
				activeParam--;
				activeParam&=3;
			}
			if ( keys & PAD_DOWN )
			{
				activeParam++;
				activeParam&=3;
			}
			if ( keys & PAD_BUTTONA )
			{
				param[activeParam]++;
			}
			if ( keys & PAD_BUTTONB )
			{
				param[activeParam]--;
			}
		}
	}
}

