/*
 * main.c Torn Desktop Demo (c) copyright Mike Wynn 2001 [03 Oct 2001]
 */

#include "gba.h"
#include "gba_io.h"
#include "gba_sprites.h"
#include "gba_font.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

extern signed short cosTable[360];
extern signed short sinTable[360];

static int param[4];
static unsigned char * digits = "0123456789ABCDEF";

static signed short getSin( int angle )
{
	while( angle < 0 ) angle+=360;
	return sinTable[angle%360];
}

static signed short getCos( int angle )
{
	while( angle < 0 ) angle+=360;
	return cosTable[angle%360];
}

static void
printNumber( int num, int x, int y, int palette )
{
	unsigned char tmp[6];
	tmp[0] = digits[(num>>12)&0xF];
	tmp[1] = digits[(num>>8)&0xF];
	tmp[2] = '.';
	tmp[3] = digits[(num>>4)&0xF];
	tmp[4] = digits[num&0xF];
	tmp[5] = 0;
	msgPrint( 0, x, y, palette, tmp ); 
}

static void
printInt( int num, int x, int y, int palette )
{
	unsigned char tmp[5];
	tmp[0] = digits[(num>>12)&0xF];
	tmp[1] = digits[(num>>8)&0xF];
	tmp[2] = digits[(num>>4)&0xF];
	tmp[3] = digits[num&0xF];
	tmp[4] = 0;
	msgPrint( 0, x, y, palette, tmp ); 
}

int main()
{
	int angle = 0;
	int tmp   = 0;

//	createTrigTable();

	wait4Vsync(); // wait until its safe to begin writing to vRam
	initialiseBG( 0, 0x0000, 0x4000, BG_TEXT_32x32, BG_COLOUR_16 );

	zeroOam();

	clearMap( 0x0000, 1024*16, 0 ); /* clear all vram */
	/* font data only starts at tile 32 */
	loadTilesFromRom( 0x4000+(32*32), l8fontData, 32*16*32 );
	loadSpriteDataFromRom( l8spriteData );
	loadBGPalette( l8fontPalette, 0, 256 );
	loadOBJPalette( l8palette, 0, 256 );

	/* enable BG0 and Objects (sprites) */
	initialiseMode( 0, 1, 0, 0, 0, 1, 0 );

	msgPrint( 0, 1, 16, 0, "hello world!" ); 

	msgPrint( 0, 1, 17, 0, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" ); 
	msgPrint( 0, 1, 18, 0, "abcdefghijklmnopqrstuvwxyz" ); 
	msgPrint( 0, 1, 19, 0, "!@#$%^&*_+|~-=\\`" );
	msgPrint( 0, 1, 20, 0, "(){}[]<>:;'\",./?" ); 

	OBJ_DATA[0].y    = (48 | OBJ_ROTATE | OBJ_SDOUBLE);
	OBJ_DATA[0].x    = (88 | OBJ_SIZE_64); // rs offset of 0;
	OBJ_DATA[0].tile =  (6<<12)|272;


	// initialise as no scale no rotate

	OBJ_ROT_DATA[0].rotParamA = param[0] = 256;
	OBJ_ROT_DATA[0].rotParamB = param[1] = 0;
	OBJ_ROT_DATA[0].rotParamC = param[2] = 0;
	OBJ_ROT_DATA[0].rotParamD = param[3] = 256;


	wait4Vsync(); // wait until its safe to begin writing to vRam

	while(1)
	{
		int keys;
		wait4Vsync(); // wait until its safe to begin writing to vRam

		msgPrint( 0, 2, 4, 0, "rot A                   " ); 
		msgPrint( 0, 2, 5, 0, "Rot B                   " ); 
		msgPrint( 0, 2, 6, 0, "Rot C                   " ); 
		msgPrint( 0, 2, 7, 0, "Rot D                   " ); 
		printNumber( param[0], 8, 4, 0 );
		printNumber( param[1], 8, 5, 0 );
		printNumber( param[2], 8, 6, 0 );
		printNumber( param[3], 8, 7, 0 );
		msgPrint( 0, 2, 9, 0, "Rotation                   " ); 
		printInt( angle, 11, 9, 0 );

		OBJ_ROT_DATA[0].rotParamA = param[0] = getCos( angle );
		OBJ_ROT_DATA[0].rotParamB = param[1] = getSin( angle );
		OBJ_ROT_DATA[0].rotParamC = param[2] = -getSin( angle );
		OBJ_ROT_DATA[0].rotParamD = param[3] = getCos( angle );
		
		keys = readKeys();
		if ( keys )
		{
			if ( keys & PAD_BUTTONA )
			{
				angle++;
			}
			if ( keys & PAD_BUTTONB )
			{
				angle--;
			}
		}
	}
}

signed short sinTable[360] = {
0,
4,
8,
13,
17,
22,
26,
31,
35,
40,
44,
48,
53,
57,
61,
66,
70,
74,
79,
83,
87,
91,
95,
100,
104,
108,
112,
116,
120,
124,
128,
131,
135,
139,
143,
146,
150,
154,
157,
161,
164,
167,
171,
174,
177,
181,
184,
187,
190,
193,
196,
198,
201,
204,
207,
209,
212,
214,
217,
219,
221,
223,
226,
228,
230,
232,
233,
235,
237,
238,
240,
242,
243,
244,
246,
247,
248,
249,
250,
251,
252,
252,
253,
254,
254,
255,
255,
255,
255,
255,
255,
255,
255,
255,
255,
255,
254,
254,
253,
252,
252,
251,
250,
249,
248,
247,
246,
244,
243,
242,
240,
238,
237,
235,
233,
232,
230,
228,
226,
223,
221,
219,
217,
214,
212,
209,
207,
204,
201,
198,
196,
193,
190,
187,
184,
181,
177,
174,
171,
167,
164,
161,
157,
154,
150,
146,
143,
139,
135,
131,
127,
124,
120,
116,
112,
108,
104,
100,
95,
91,
87,
83,
79,
74,
70,
66,
61,
57,
53,
48,
44,
40,
35,
31,
26,
22,
17,
13,
8,
4,
0,
65532,
65528,
65523,
65519,
65514,
65510,
65505,
65501,
65496,
65492,
65488,
65483,
65479,
65475,
65470,
65466,
65462,
65457,
65453,
65449,
65445,
65441,
65436,
65432,
65428,
65424,
65420,
65416,
65412,
65408,
65405,
65401,
65397,
65393,
65390,
65386,
65382,
65379,
65375,
65372,
65369,
65365,
65362,
65359,
65355,
65352,
65349,
65346,
65343,
65340,
65338,
65335,
65332,
65329,
65327,
65324,
65322,
65319,
65317,
65315,
65313,
65310,
65308,
65306,
65304,
65303,
65301,
65299,
65298,
65296,
65294,
65293,
65292,
65290,
65289,
65288,
65287,
65286,
65285,
65284,
65284,
65283,
65282,
65282,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65282,
65282,
65283,
65284,
65284,
65285,
65286,
65287,
65288,
65289,
65290,
65292,
65293,
65294,
65296,
65298,
65299,
65301,
65303,
65304,
65306,
65308,
65310,
65313,
65315,
65317,
65319,
65322,
65324,
65327,
65329,
65332,
65335,
65338,
65340,
65343,
65346,
65349,
65352,
65355,
65359,
65362,
65365,
65369,
65372,
65375,
65379,
65382,
65386,
65390,
65393,
65397,
65401,
65405,
65409,
65412,
65416,
65420,
65424,
65428,
65432,
65436,
65441,
65445,
65449,
65453,
65457,
65462,
65466,
65470,
65475,
65479,
65483,
65488,
65492,
65496,
65501,
65505,
65510,
65514,
65519,
65523,
65528,
65532
};

static signed short cosTable[360] = {
256,
255,
255,
255,
255,
255,
254,
254,
253,
252,
252,
251,
250,
249,
248,
247,
246,
244,
243,
242,
240,
238,
237,
235,
233,
232,
230,
228,
226,
223,
221,
219,
217,
214,
212,
209,
207,
204,
201,
198,
196,
193,
190,
187,
184,
181,
177,
174,
171,
167,
164,
161,
157,
154,
150,
146,
143,
139,
135,
131,
127,
124,
120,
116,
112,
108,
104,
100,
95,
91,
87,
83,
79,
74,
70,
66,
61,
57,
53,
48,
44,
40,
35,
31,
26,
22,
17,
13,
8,
4,
0,
65532,
65528,
65523,
65519,
65514,
65510,
65505,
65501,
65496,
65492,
65488,
65483,
65479,
65475,
65470,
65466,
65462,
65457,
65453,
65449,
65445,
65441,
65436,
65432,
65428,
65424,
65420,
65416,
65412,
65408,
65405,
65401,
65397,
65393,
65390,
65386,
65382,
65379,
65375,
65372,
65369,
65365,
65362,
65359,
65355,
65352,
65349,
65346,
65343,
65340,
65338,
65335,
65332,
65329,
65327,
65324,
65322,
65319,
65317,
65315,
65313,
65310,
65308,
65306,
65304,
65303,
65301,
65299,
65298,
65296,
65294,
65293,
65292,
65290,
65289,
65288,
65287,
65286,
65285,
65284,
65284,
65283,
65282,
65282,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65281,
65282,
65282,
65283,
65284,
65284,
65285,
65286,
65287,
65288,
65289,
65290,
65292,
65293,
65294,
65296,
65298,
65299,
65301,
65303,
65304,
65306,
65308,
65310,
65313,
65315,
65317,
65319,
65322,
65324,
65327,
65329,
65332,
65335,
65338,
65340,
65343,
65346,
65349,
65352,
65355,
65359,
65362,
65365,
65369,
65372,
65375,
65379,
65382,
65386,
65390,
65393,
65397,
65401,
65405,
65409,
65412,
65416,
65420,
65424,
65428,
65432,
65436,
65441,
65445,
65449,
65453,
65457,
65462,
65466,
65470,
65475,
65479,
65483,
65488,
65492,
65496,
65501,
65505,
65510,
65514,
65519,
65523,
65528,
65532,
0,
4,
8,
13,
17,
22,
26,
31,
35,
40,
44,
48,
53,
57,
61,
66,
70,
74,
79,
83,
87,
91,
95,
100,
104,
108,
112,
116,
120,
124,
128,
131,
135,
139,
143,
146,
150,
154,
157,
161,
164,
167,
171,
174,
177,
181,
184,
187,
190,
193,
196,
198,
201,
204,
207,
209,
212,
214,
217,
219,
221,
223,
226,
228,
230,
232,
233,
235,
237,
238,
240,
242,
243,
244,
246,
247,
248,
249,
250,
251,
252,
252,
253,
254,
254,
255,
255,
255,
255,
255
};


